/*
Cascading style sheet for the Durus documentation.
Just overrides what I don't like about the standard docutils
stylesheet.

$Id: default.css 20217 2003-01-16 20:51:53Z akuchlin $
*/

@import url(/misc/docutils.css);

pre.literal-block, pre.doctest-block {
  margin-left: 1em ;
  margin-right: 1em ;
  background-color: #f4f4f4 }

tt { background-color: transparent }
